﻿' **********************************************
' TSND121にSensorServer経由で計測開始/停止を送るだけの簡単なサンプルPG2(Visual BASIC)
'
' 1.SensorServerを適切に設定し，実行（待ち受け状態に）して下さい
'
' 2.本プログラム実行前に，TCPポート番号を適切な値に変更して下さい
'
' 3.予め計測設定をSensorControllerで行っておくか，"sendStrCommand"を
'   参考に計測設定を行うように設定してから実行して下さい
'
' 本プログラムは，SensorServer経由でコマンドを送信する簡単なサンプル
' プログラムです．本来は，実行したコマンドのレスポンスを確認して
' エラーの有無をチェックしたり，受信データをハンドリングする
' 処理部が必要になります．また，十分なエラー処理を行っていませんので，
' エラー発生時には意図せず終了してしまう場合があります．
'
' Copyright (C) 2013  ATR-Promotinos, Inc.
'
' 2013/01/15 T.Adachi
' 2013/05/02 modified by T.Adachi
' **********************************************

Imports System
Imports System.Net
Imports System.Net.Sockets
Imports System.Threading
Imports System.Text

Public Class VBSample2Form

    ' SensorServerの動いているホスト名と
    Dim hostName As String = "localhost"
    ' 使用するTSND121に対応したポート番号
    Dim portNum As Integer = 11000

    Dim tcpClient As New System.Net.Sockets.TcpClient
    Dim thread As System.Threading.Thread


    Private Shared connectDone As New ManualResetEvent(False)
    Private Shared sendDone As New ManualResetEvent(False)
    Private Shared receiveDone As New ManualResetEvent(False)


    ' 接続・初期設定
    Private Sub ConnectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ConnectButton.Click
        Try
            ' 接続
            tcpClient.Connect(hostName, portNum)
            ' 接続待ち
            Do While tcpClient.Connected = False
                System.Threading.Thread.Sleep(500)
            Loop

            ' 受信用スレッドを開始
            thread = New System.Threading.Thread(New ParameterizedThreadStart(AddressOf AsyncReadCallbackWorker))
            thread.Start(tcpClient)

            ' センサ設定
            ' 加速度・角速度を20ms, 1avg, BT送信, 書き込み無しで初期化
            Dim sendStrCommand As String = "setags 20 1 0" & Chr(13)
            Dim sendBytesCommand As Byte() = System.Text.Encoding.ASCII.GetBytes(sendStrCommand)

            ' 送信
            tcpClient.GetStream.Write(sendBytesCommand, 0, sendBytesCommand.Length)

        Catch ex As Exception
            ' エラー発生接続出来なかった

            Return
        End Try

        ConnectButton.Enabled = False
        StartButton.Enabled = True
        StopButton.Enabled = False
        DisconnectButton.Enabled = True

    End Sub

    ' 切断
    Private Sub DisconnectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DisconnectButton.Click
        Try

            thread.Abort()
            tcpClient.GetStream.Close()
            tcpClient.Close()

            ' 次回接続用に確保
            tcpClient = New System.Net.Sockets.TcpClient
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Return
        End Try

        ConnectButton.Enabled = True
        StartButton.Enabled = False
        StopButton.Enabled = False
        DisconnectButton.Enabled = False
    End Sub


    ' 開始
    Private Sub StartButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StartButton.Click
        ' 送信コマンドをByte列に変換
        Dim sendStrCommand As String = "start" & Chr(13)
        Dim sendBytesCommand As Byte() = System.Text.Encoding.ASCII.GetBytes(sendStrCommand)

        Try
            ' 送信
            tcpClient.GetStream.Write(sendBytesCommand, 0, sendBytesCommand.Length)
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Return
        End Try

        StartButton.Enabled = False
        StopButton.Enabled = True

    End Sub

    ' 停止
    Private Sub StopButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StopButton.Click
        ' 送信コマンドをByte列に変換
        Dim sendStrCommand As String = "stop" & Chr(13)
        Dim sendBytesCommand As Byte() = System.Text.Encoding.ASCII.GetBytes(sendStrCommand)

        ' 送信
        tcpClient.GetStream.Write(sendBytesCommand, 0, sendBytesCommand.Length)

        StartButton.Enabled = True
        StopButton.Enabled = False
    End Sub

    ' 受信ログのクリア
    Private Sub ClearButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ClearButton.Click
        LogTextBox.Clear()
    End Sub

    ' 別スレッドからのコントロール操作用
    Delegate Sub AppendTextDelegate(strReadData As String)

    Sub AppendText(ByVal text As String)
        If InvokeRequired Then
            Dim d As New AppendTextDelegate(AddressOf AppendText)
            Me.Invoke(d, text)
            Return
        End If

        LogTextBox.AppendText(text + Environment.NewLine)
    End Sub

    ' 受信スレッド
    Sub AsyncReadCallbackWorker(ByVal obj As Object)
        Try
            Do While True
                Dim tcpClient As TcpClient = obj
                Dim readBytes(512 * 1024) As Byte
                Dim readSize As Integer
                Dim ms As New System.IO.MemoryStream

                ' 受信バッファにあるデータを一通り読み出す
                Do
                    ' 受信
                    readSize = tcpClient.GetStream.Read(readBytes, 0, readBytes.Length)

                    ' 終了判定
                    If readSize = 0 Then
                        Exit Do
                    End If
                    ms.Write(readBytes, 0, readSize)
                Loop While tcpClient.GetStream.DataAvailable

                '受信したUnicodeデータを文字列に変換
                Dim strReadData As String = Encoding.Unicode.GetString(ms.ToArray())
                ' Console.WriteLine(strReadData)
                ms.Close()

                ' 受信テキスト追記
                AppendText(strReadData)

                ' 待ち
                thread.Sleep(10)
            Loop
        Catch ex As Exception
            'MessageBox.Show(ex.Message)
        End Try
    End Sub

End Class
